@@ -59,6 +59,8 @@ modified by
 #include "netinet_any.h"
 #include "packetfilter_api.h"
 
+namespace srt {
+
 //////////////////////////////////////////////////////////////////////////////
 // The purpose of the IOVector class is to proide a platform-independet interface
 // to the WSABUF on Windows and iovec on Linux, that can be easilly converted
@@ -214,8 +216,6 @@ inline EncryptionKeySpec GetEncryptionKeySpec(int32_t msgno)
 
 const int32_t PUMASK_SEQNO_PROBE = 0xF;
 
-
-namespace srt {
 std::string PacketMessageFlagStr(uint32_t msgno_field);
 
 class CPacket
@@ -284,7 +284,7 @@ public:
 
    void setControl(UDTMessageType type)
    {
-       m_nHeader[srt::SRT_PH_SEQNO] = SEQNO_CONTROL::mask | SEQNO_MSGTYPE::wrap(type);
+       m_nHeader[SRT_PH_SEQNO] = SEQNO_CONTROL::mask | SEQNO_MSGTYPE::wrap(type);
    }
 
       /// Read the extended packet type.
