@@ -54,10 +54,12 @@
 #include "utilities.h"
 #include "srt_attr_defs.h"
 
-class CUDTException;    // defined in common.h
 
 namespace srt
 {
+
+class CUDTException;    // defined in common.h
+
 namespace sync
 {
 
@@ -344,7 +346,7 @@ class SRT_ATTR_SCOPED_CAPABILITY ScopedLock
 {
 public:
     SRT_ATTR_ACQUIRE(m)
-    ScopedLock(Mutex& m);
+    explicit ScopedLock(Mutex& m);
 
     SRT_ATTR_RELEASE()
     ~ScopedLock();
@@ -357,27 +359,25 @@ private:
 class SRT_ATTR_SCOPED_CAPABILITY UniqueLock
 {
     friend class SyncEvent;
+    int m_iLocked;
+    Mutex& m_Mutex;
 
 public:
-    SRT_ATTR_ACQUIRE(m_Mutex)
-    UniqueLock(Mutex &m);
+    SRT_ATTR_ACQUIRE(m)
+    explicit UniqueLock(Mutex &m);
 
-    SRT_ATTR_RELEASE(m_Mutex)
+    SRT_ATTR_RELEASE()
     ~UniqueLock();
 
 public:
-    SRT_ATTR_ACQUIRE(m_Mutex)
+    SRT_ATTR_ACQUIRE()
     void lock();
 
-    SRT_ATTR_RELEASE(m_Mutex)
+    SRT_ATTR_RELEASE()
     void unlock();
 
     SRT_ATTR_RETURN_CAPABILITY(m_Mutex)
     Mutex* mutex(); // reflects C++11 unique_lock::mutex()
-
-private:
-    int m_iLocked;
-    Mutex& m_Mutex;
 };
 #endif // ENABLE_STDCXX_SYNC
 
@@ -743,7 +743,7 @@ typedef std::system_error CThreadException;
 using CThread = std::thread;
 namespace this_thread = std::this_thread;
 #else // pthreads wrapper version
-typedef ::CUDTException CThreadException;
+typedef CUDTException CThreadException;
 
 class CThread
 {
