@@ -1,8 +1,8 @@
 #include "gtest/gtest.h"
+#include <array>
 #include <chrono>
 #include <thread>
 #include <future>
-#include <array>
 #include <numeric> // std::accumulate
 #include <regex>   // Used in FormatTime test
 #include "sync.h"
@@ -15,14 +15,6 @@
 using namespace std;
 using namespace srt::sync;
 
-// GNUC supports C++14 starting from version 5
-//#if defined(__GNUC__) && (__GNUC__ < 5)
-////namespace srt
-// constexpr chrono::milliseconds operator"" ms(
-//    unsigned long long _Val) { // return integral milliseconds
-//    return chrono::milliseconds(_Val);
-//}
-//#endif
 
 TEST(SyncDuration, BasicChecks)
 {
@@ -159,23 +151,44 @@ TEST(SyncDuration, OperatorMultIntEq)
 
 TEST(SyncRandom, GenRandomInt)
 {
-    vector<int> mn(64);
+    array<int, 64> mn = {};
 
-    for (int i = 0; i < 2048; ++i)
+    // Check generated values are in the specified range.
+    const size_t n = 2048;
+    for (size_t i = 0; i < n; ++i)
     {
-        const int rand_val = genRandomInt(0, 63);
+        const int rand_val = genRandomInt(0, mn.size() - 1);
         ASSERT_GE(rand_val, 0);
-        ASSERT_LE(rand_val, 63);
+        ASSERT_LT(rand_val, mn.size());
         ++mn[rand_val];
     }
 
+    // Check the distribution is more or less uniform.
+    // 100% uniform if each value is generated (n / (2 * mn.size())) times.
+    // We expect at least half of that value for a random uniform distribution.
+    const int min_value = n / (2 * mn.size()) - 1;
+    cout << "min value: " << min_value << endl;
+    for (size_t i = 0; i < mn.size(); ++i)
+    {
+        EXPECT_GE(mn[i], min_value) << "i=" << i << ". Ok-ish if the count is non-zero.";
+    }
+
     // Uncomment to see the distribution.
-    // for (size_t i = 0; i < mn.size(); ++i)
-    // {
-    //     cout << i << '\t';
-    //     for (int j=0; j<mn[i]; ++j) cout << '*';
-    //     cout << '\n';
-    // }
+    //for (size_t i = 0; i < mn.size(); ++i)
+    //{
+    //    cout << i << '\t';
+    //    for (int j=0; j<mn[i]; ++j) cout << '*';
+    //    cout << '\n';
+    //}
+
+    // Check INT32_MAX
+    for (size_t i = 0; i < n; ++i)
+    {
+        const int rand_val = genRandomInt(INT32_MAX - 1, INT32_MAX);
+
+        EXPECT_GE(rand_val, INT32_MAX - 1);
+        EXPECT_LE(rand_val, INT32_MAX);
+    }
 }
 
 /*****************************************************************************/
@@ -416,7 +429,7 @@ TEST(SyncEvent, WaitForTwoNotifyOne)
     const int VAL_SIGNAL = 42;
     const int VAL_NO_SIGNAL = 0;
 
-    volatile bool resource_ready = true;
+    srt::sync::atomic<bool> resource_ready(true);
 
     auto wait_async = [&](Condition* cond, Mutex* mutex, const steady_clock::duration& timeout, int id) {
         UniqueLock lock(*mutex);
@@ -580,7 +593,7 @@ void* dummythread(void* param)
 TEST(SyncThread, Joinable)
 {
     CThread foo;
-    volatile bool thread_finished = false;
+    srt::sync::atomic<bool> thread_finished;
 
     StartThread(foo, dummythread, (void*)&thread_finished, "DumyThread");
 
